// https://typst.app/universe/package/touying/
// https://github.com/touying-typ/touying
#import "@preview/touying:0.6.0": *
#import themes.university: *
#import "@preview/cetz:0.3.1"
#import "@preview/fletcher:0.5.3" as fletcher: node, edge
#import "@preview/ctheorems:1.1.3": *
#import "@preview/numbly:0.1.0": numbly

// cetz and fletcher bindings for touying
#let cetz-canvas = touying-reducer.with(reduce: cetz.canvas, cover: cetz.draw.hide.with(bounds: true))
#let fletcher-diagram = touying-reducer.with(reduce: fletcher.diagram, cover: fletcher.hide)

// Theorems configuration by ctheorems
#show: thmrules.with(qed-symbol: $square$)
#let theorem = thmbox("theorem", "Theorem", fill: rgb("#eeffee"))
#let corollary = thmplain(
  "corollary",
  "Corollary",
  base: "theorem",
  titlefmt: strong,
)
#let definition = thmbox("definition", "Definition", inset: (x: 1.2em, top: 1em))
#let example = thmplain("example", "Example").with(numbering: none)
#let proof = thmproof("proof", "Proof")

#show: university-theme.with(
  aspect-ratio: "16-9",
  //   config-common(handout: true),
  config-info(
    title: [Typst Cheat Sheet],
    subtitle: [Using #link("https://touying-typ.github.io/zh/")[Touying]],
    author: [by #link("https://darstib.github.io/")[Darstib]],
    date: datetime.today(),
    // institution: [personal],
    logo: emoji.school,
  ),
)

// set heading
#set heading(numbering: numbly("{1}.", default: "1.1"))
// set table
#show table.cell.where(y: 0): strong
#set table(
  columns: (auto, auto, auto),
  stroke: (x, y) => if y == 0 {
    (bottom: 0.7pt + black)
  },
  align: (x, y) => (
    if x > 0 { center } else { left }
  ),
)

/* Content begin */

#title-slide()

== Outline <touying:hidden>

#components.adaptive-columns(max-count: 2, outline(title: none, indent: 1em))

= #link("https://typst.app/docs/reference/syntax/")[Syntax]

== Modes<modes>

#table(
  // columns: (auto, auto, auto),
  inset: 8pt,
  align: (center + horizon),
  [*New mode*], [*Syntax*], [*Example*],
  [Code], [Prefix the code with `#`], [Number: `#(1 + 2)`],
  [Math], [Surround equation with `$..$`], [`$-x$` is the opposite of `$x$`],
  [Markup], [Surround markup with `[..]`], [#raw("#let name = [*Typst!*]", lang: "typst")],
)

== #link("https://typst.app/docs/reference/syntax/#markup")[Markup]

#slide(composer: (1fr, 1fr))[
  Typst is a markup language. This means that you can use simple syntax to accomplish common layout tasks.
  #parbreak() // equal to a blank line
  - symbol
    + *strong*
    + _emphasis_
    + `raw text`
    + `<label>` and `@reference`
    + /* comment */ // comment
][

  - function
    + #strong[strong]
    + #emph[emphasis]
    + #raw("fn main();", lang: "rust")
    + #link("https://typst.app/docs/reference/text/raw/")[Link to raw usage]
    + #overline[overline]
    + #underline[underline]
    + #highlight[highlight]
    + #strike[strike]
    + #footnote[this is a footnote]<fn>
]

== Math

A in line function: $a^2 + b^2 = c^2$

A block function:

$ e^(i pi) + 1 = 0 $

more complex:

$ Mu:= mat(1, 2, 3, 4, 5, 6, 7, 8, 9) $

#link("https://typst.app/docs/reference/symbols/sym/")[*More symbol*]

== #link("https://typst.app/docs/reference/syntax/#code")[Code]

#slide[
  *Data type:*
  - Length: 1em, 2pt, 3mm
  - Angle: 1deg, 2rad, 3grad
  - Fraction: 1fr
  - Ratio: 50%
  - Array: (1, 2, 3)
  - Dictionary: (a: 1, b: 2, c: "hi")
  - Content block: `[content]`
  - Code block: { let x =1; x+1}

][
  *Usage:*
  - Field access: field.name
  - Function call: function()
  - Arg spreading: function(..args)
  - Method call: field.method()
  - Unnamed func: #raw("(a,b)=>a+b", lang: "rust")
  - Named: #raw("let add(a,b)=a+b", lang: "rust")
  - Let blinding: `#let a = 1`
  - #link("http://typst.app/docs/reference/styling/")[`#set` and `#show`]
  - #link("https://typst.app/docs/reference/context/")[`# context`]
]

= Get deeper

== #link("https://typst.app/docs/reference/scripting/")[Scripting]

#slide[

  === Field
  ```typst
  #let it = [== subtitle]
  #let dict = (greet: "Hello")
  #it.fields()
  #dict.greet
  #emoji.face
  ```
][\ \ \
  #let it = [== subtitle]
  #let dict = (greet: "Hello")
  // #dict.fields() // type dictionary has no method `fields`
  #it.fields()
  #dict.greet \
  #emoji.face
]

#slide[
  === Method

  Just like in python
  ```typst
  #let demo_str = "Hello, typst!"
  #demo_str
  #demo_str.len()
  #str.len(demo_str)
  ```
][
  \ \ \ \
  #let demo_str = "Hello, typst!"
  #demo_str \
  #demo_str.len() \
  #str.len(demo_str)
]

#slide[
  === Flow

  *Loop*


  ```typ
  #let n = 2
  #while n < 10 {
      n = (n * 2) - 1
      (n,)
  }

  #let s = "Hello, typst!"
  #for char in s [
      (#char,)
  ]
  ```
][
  Just like in python\
  *Condition*

  ```typ
  #if 1 < 2 [
    This is shown
  ] else [
    This is not.
  ]
  ```
]

== #link("https://typst.app/docs/reference/styling/")[Styling]

== #link("https://typst.app/docs/reference/foundations/selector/")[Selector]

#slide[
  === constructor

  - *func element*: `heading` `figure`
  - *special field*: `self.where(..any)`
  - *regex*: `^[a-z]+`
  - *location*: `#here().page()`
  - *`<lable>`*
][
  === definition

  - *self.or*(selector)
  - *self.and*(selector)
  - *self.before*(selector)
  - *self.after*(selector)
]

== #link("https://typst.app/docs/reference/foundations/function/")[Function]

#let alert(body, fill: red, inset: 8pt, radius: 4pt) = {
  set text(white)
  set align(center)
  rect(
    fill: fill,
    inset: inset,
    radius: radius,
    [*Warning:\ #body*],
  )
}

#slide[

  === define<inside-title>

  Using `#let` blinding to define a function, with a code block as the body.
  // #pause
  #alert(fill: blue)[
    This is a warning message.
  ]
][
  ```typst
  #let alert(body, fill: red, inset: 8pt, radius: 4pt) = {
    set text(white)
    set align(center)
    rect(
      fill: fill,
      inset: inset,
      radius: radius,
      [*Warning:\ #body*],
    )
  }
  ```
]

=== Import

Functions can be imported from one file (module) into another using import. For example, assume that we have defined the alert function from the previous example in a file called foo.typ. We can import it into another file by writing #raw("import 'foo.typ': alert.", lang: "t")

== #link("https://typst.app/docs/reference/model/figure/")[Figure]

#show figure: set block(breakable: false)
#show figure.caption: emph
#show figure.where(kind: table): set figure.caption(position: top)

=== #link("https://typst.app/docs/reference/visualize/image/")[image]

#figure(
  image("imgs/demo1.png", alt: "kamisato", width: 40%),
  caption: [Kamisato Ayaka],
  //   placement: auto,
  //   kind: image,
)<demo1>

The @demo1 is a demo picture.

=== #link("https://typst.app/docs/reference/model/table/")[table]



#figure(
  table(
    columns: (auto, auto, auto),
    inset: 8pt,
    align: horizon,
    table.header(
      [],
      [*Volume*],
      [*Parameters*],
    ),

    [function1],
    $ pi h (D^2 - d^2) / 4 $,
    [
      $h$: height \
      $D$: outer radius \
      $d$: inner radius
    ],

    [function2], $ sqrt(2) / 12 a^3 $, [$a$: edge length],
  ),
  caption: [Timing results],
)<timeresult>

The @timeresult is a demo table.

== visualize

#slide(composer: (2fr, 1fr))[

  === basic graph
  ```typ
  - #circle(radius: 10pt, fill: orange)
  - #rect(height: 20pt, fill: blue)
  - #square with gradient
  - #ellipse(height: 20pt, fill: green)
  - #polygon
  - #line
  ```
][\
  - #circle(radius: 10pt, fill: orange)
  - #rect(height: 15pt, fill: blue)
    #set square(height: 20pt)
  - #stack(
      dir: ltr,
      spacing: 1fr,
      square(fill: gradient.linear(..color.map.rainbow)),
      square(fill: gradient.radial(..color.map.rainbow)),
      square(fill: gradient.conic(..color.map.rainbow)),
    )
  - #ellipse(height: 20pt, fill: green)
  - #polygon(
      fill: blue.lighten(80%),
      stroke: blue,
      (20%, 0pt),
      (60%, 0pt),
      (80%, 20pt),
      (0%, 20pt),
    )
  - #stack(
      dir: ltr,
      spacing: 1em,
      line(stroke: 2pt + red),
      line(stroke: (paint: blue, thickness: 4pt, cap: "round")),
      line(stroke: (paint: blue, thickness: 1pt, dash: "dashed")),
      line(stroke: 2pt + gradient.linear(..color.map.rainbow)),
    )
]


== Other

#slide[
  === Sinks & Spreading
  #v(10pt)
  We can specify an argument sink which collects all excess arguments as `..args` and just spread it with `..args`
  #line(length: 100%, stroke: 0.1pt)
  #let format(title, ..authors) = {
    let by = authors.pos().join(", ", last: " and ")
    [*#title* \ _Written by #by;_]
  }
  #format("ArtosFlow", "Jane", "Joe", "Jake")
  #let arr = (1, 3, 5, 7, 9)
  #calc.max(..arr)
][
  ```typ
  #let format(title, ..authors) = {
    let by = authors.pos()
      .join(", ", last: " and ")
    [*#title* \ _Written by #by;_]
  }
  #format("ArtosFlow", "Jane", "Joe", "Jake")
  #let arr = (1, 3, 5, 7, 9)
  #calc.min(..arr)
  ```
]

#slide[
  === #link("https://typst.app/docs/reference/foundations/regex/")[Regex]
  ```typ
  // Works with string methods.
  #"a,b;c".split(regex("[,;]"))

  // Works with show rules.
  #show regex("\d+"): set text(red)

  The numbers 1 to 10.
  ```
][\ \
  // Works with string methods.
  #"a,b;c".split(regex("[,;]"))
  // Works with show rules.
  #show regex("\d+"): set text(red)
  \ \ \ \ \
  The numbers 1 to 10.
]

=== #link("https://typst.app/docs/reference/model/quote/")[Quote]

#set quote(block: true)
#show quote: set pad(x: 3em)
// #show quote: set align(center)

#quote(attribution: [Plato])[
  ... ἔοικα γοῦν τούτου γε σμικρῷ τινι αὐτῷ τούτῳ σοφώτερος εἶναι, ὅτι
  ἃ μὴ οἶδα οὐδὲ οἴομαι εἰδέναι.
]


= Animation

== Simple Animation

We can use `#pause` to #pause display something later.#pause Just like this.

#meanwhile

Meanwhile, #pause we can also use `#meanwhile` to #pause display other content synchronously.

#speaker-note[
  + This is a speaker note.
  + You won't see it unless you use `config-common(show-notes-on-second-screen: right)`
]


== Complex Animation

At subslide #touying-fn-wrapper((self: none) => str(self.subslide)), we can

use #uncover("2-")[`#uncover` function] for reserving space,

use #only("2-")[`#only` function] for not reserving space,

#alternatives[call `#only` multiple times \u{2717}][use `#alternatives` function #sym.checkmark] for choosing one of the alternatives.


== Callback Style

#slide(
  repeat: 3,
  self => [
    #let (uncover, only, alternatives) = utils.methods(self)

    At subslide #self.subslide, we can

    use #uncover("2-")[`#uncover` function] for reserving space,

    use #only("2-")[`#only` function] for not reserving space,

    #alternatives[call `#only` multiple times \u{2717}][use `#alternatives` function #sym.checkmark] for choosing one of the alternatives.
  ],
)


== Math Equation

Equation with `pause`:

$
  f(x) &= pause x^2 + 2x + 1 \
  &= pause (x + 1)^2 \
$

#meanwhile

Here, #pause we have the expression of $f(x)$.

#pause

By factorizing, we can obtain this result.


== CeTZ Animation

CeTZ Animation in Touying:

#cetz-canvas({
  import cetz.draw: *

  rect((0, 0), (5, 5))

  (pause,)

  rect((0, 0), (1, 1))
  rect((1, 1), (2, 2))
  rect((2, 2), (3, 3))

  (pause,)

  line((0, 0), (2.5, 2.5), name: "line")
})


== Fletcher Animation

Fletcher Animation in Touying:

#fletcher-diagram(
  node-stroke: .1em,
  node-fill: gradient.radial(blue.lighten(80%), blue, center: (30%, 20%), radius: 80%),
  spacing: 4em,
  edge((-1, 0), "r", "-|>", `open(path)`, label-pos: 0, label-side: center),
  node((0, 0), `reading`, radius: 2em),
  edge((0, 0), (0, 0), `read()`, "--|>", bend: 130deg),
  pause,
  edge(`read()`, "-|>"),
  node((1, 0), `eof`, radius: 2em),
  pause,
  edge(`close()`, "-|>"),
  node((2, 0), `closed`, radius: 2em, extrude: (-2.5, 0)),
  edge((0, 0), (2, 0), `close()`, "-|>", bend: -40deg),
)


= Theorems

== Prime num

#definition[
  A natural number is called a #highlight[_prime number_] if it is greater
  than 1 and cannot be written as the product of two smaller natural numbers.
]
#example[
  The numbers $2$, $3$, and $17$ are prime.
  @cor_largest_prime shows that this list is not exhaustive!
]

#theorem("Euclid")[
  There are infinitely many primes.
]
#proof[
  Suppose to the contrary that $p_1, p_2, dots, p_n$ is a finite enumeration
  of all primes. Set $P = p_1 p_2 dots p_n$. Since $P + 1$ is not in our list,
  it cannot be prime. Thus, some prime factor $p_j$ divides $P + 1$. Since
  $p_j$ also divides $P$, it must divide the difference $(P + 1) - P = 1$, a
  contradiction.
]

#corollary[
  There is no largest prime number.
] <cor_largest_prime>
#corollary[
  There are infinitely many composite numbers.
]

#theorem[
  There are arbitrarily long stretches of composite numbers.
]

#proof[
  For any $n > 2$, consider $
    n! + 2, quad n! + 3, quad ..., quad n! + n #qedhere
  $
]


= Others

== Side-by-side

#slide(composer: (1fr, 1fr))[
  First column.
][
  Second column.
]


== Multiple Pages

#lorem(200)


#show: appendix

= Appendix

== Appendix

Please pay attention to the current slide number.
